######## loading packages
library(flowCore)
library(flowViz)
#library(flowStats) to ba added in case of problems

library(gdata)
library(Hmisc)
library(caroline)
library(sm)
library(beanplot)
library(UsingR)

#library(modeest)

##### FILTER function to remove the outliers due to previous experiments (the machine is dirty)
filter <- function(x){
  qnt <- quantile(x, probs=c(.25, .75), na.rm = TRUE)  
  H <- 2.5 * IQR(x, na.rm = TRUE)  
  flt <- x
  flt[x < (qnt[1] - H)] <- NA
  flt[x > (qnt[2] + H)] <- NA
  flt
}
#####

##### FILTER function to remove the data corresponding to negative fluorescence
filter_low <- function(x){
  flt <- x
  flt[x < 0] <- NA
  flt
}
#####

# experiment name and directories
e_name=''
dir_d='' # fcs files path
dir_r=''  # results path

# Selecting the fluorescence channel--GFP: FL1-H
# Selecting the fluorescence channel--RFP: FL3-H
selected_ch="GFP-A"
#selected_ch="Pacific Blue-A"
# Set the number of samples
s_num=

# Setting sxlim, sylim (dot plot)
sxlim=
sylim=

# Setting xlimit, ylimit for plotting histograms etc
xlimit=
ylimit=


# loading the data into a flowSet
fs <- read.flowSet(dir(dir_d, full.names=TRUE))

#### GATING
# morphGate, automatic  
morphGate <- norm2Filter("FSC-A", "SSC-A", filterId="MorphologyGate")

#### polygonGate, by hands
#mat <- matrix(c(50000,150000,125000,
#                50000,150000,25000)
#              ,nrow = 3)
#colnames(mat) <-c("FSC-A", "SSC-A")
# mat
# pg <- polygonGate(mat)

#### rectangleGate, by hands
#rg <- rectangleGate(filterId="myRectGate", "FSC-A"=c(50000, 150000), "SSC-A"=c(50000, 150000))

# gating the data into a Subset (still a flowSet)
 gated <- Subset(fs, morphGate)
# gated <- Subset(fs, pg)
#gated <- Subset(fs, rg)

res=256
# plotting the gated data -- an example
print(xyplot(`SSC-A` ~ `FSC-A`, data=gated[[1]], nbin=res, xlim=c(0,sxlim), ylim=c(0,sylim)))

# printing the whole data in a PDF file
pdfFile <- paste(dir_r,"all_",e_name,".pdf", sep="")
pdf(file=pdfFile)
print(xyplot(`SSC-A` ~ `FSC-A`, data=fs, nbin=res, xlim=c(0,sxlim), ylim=c(0,sylim)))
dev.off()

# printing the gated data in a PDF file
pdfFile <- paste(dir_r,"gated_",e_name,".pdf", sep="")
pdf(file=pdfFile)
print(xyplot(`SSC-A` ~ `FSC-A`, data=gated, nbin=res, xlim=c(0,sxlim), ylim=c(0,sylim)))
dev.off()

#### 
xvalues <- c(1:s_num)
print('xvalues')
no_names <- rep('', s_num)
p_labels <- c()

# Extracting one channel only from every file:
channels <- colnames(gated)       # measured quantities
files <- sampleNames(gated)       # facs file names

print(channels)
print(files)

#### FULL ANALYSIS
#### Converting the data from FlowSet to DataFrame
name <- files[[1]]
p_labels[1] <- name 
df_s <- as.data.frame((gated[[name]]@exprs[,selected_ch]))

for (i in 2:length(files)){
    name <- files[[i]]
    p_labels[i] <- name
    print(name)
    new_fcs <- as.data.frame((gated[[name]]@exprs[,selected_ch]))
    df_s <- cbindX(df_s,new_fcs)
}

colnames(df_s) <- files

#### Visualizing all the data 
print(apply(df_s, 2, summary))  # 2 stands for the columns of the data frame

######## extracting mean etc.
meanr<- apply(df_s, 2, mean, na.rm=TRUE)          # mean
varr<- apply(df_s, 2, var, na.rm=TRUE)            # variance
sigmar <- c()                                     # standard deviation

###### Writing all important information on two text file (full_analysis.txt and full_table.txt)
tfile <- paste(dir_r,"full_table.txt")
write(e_name,tfile,sep="\t",append=TRUE)

wfile <- paste(dir_r,"full_analysis.txt")
write(e_name,wfile,sep="\t",append=TRUE)

head='Sample\tmean\tst. dev.'
write(head, tfile, ncolumns=1, sep="\t",append=TRUE) 
for (i in 1:length(files)){
  write(colnames(df_s[i]),wfile,sep='\t',append=TRUE)
  l1="mean:"
  write(l1,wfile,sep="\t",append=TRUE)
  write(meanr[i],wfile,sep="\t",append=TRUE)
  sigmar[i]=sqrt(varr[i])
  l2="err:"
  write(l2,wfile,sep="\t",append=TRUE)
  write(sigmar[i],wfile,sep="\t",append=TRUE)
  write(summary(df_s[i]),wfile,sep="\t",append=TRUE)
  lines <- matrix( c(colnames(df_s[i]), meanr[i], sigmar[i]), nrow=1, ncol=3)
  write(lines, tfile, ncolumns=3, sep="\t",append=TRUE)    
  write('',wfile,sep="\t",append=TRUE)
}  

# Defining the barplot error bars
err_bars_pf <- meanr+sigmar
err_bars_mf <- meanr-sigmar


### Visualizing the outliers
# coord is a matrix whose first (and only) column contains the x axis coordinates where the barplots are placed
# barplots will be printed later, here we need only to initialize coord matrix for boxplot
coord <- barplot(meanr,xlim=c(0,xlimit),ylim=c(0,ylimit),ylab="Fluorescence (AU)",names.arg=no_names,main=e_name)


########### BOXPLOTS 
### The box is given by 75% quantile, median, and 25% quantile.
### The bars are calculated by multipling the interquartile distance (75% qnt - 25% qnt) times 1.5 ("range" default value)
### The cyrcles outside the bars are outliers
pdfFile <- paste(dir_r,"boxplots.pdf", sep="")
pdf(file=pdfFile)
boxplot(df_s, ylim=c(0,ylimit),xlim=c(0.5,xlimit), at=coord[,1], show.names=F, varwidth=T, outline=T, main=e_name, ylab="Fluorescence (AU)",names.arg=no_names)
text(coord[,1], par("usr")[3], srt = 60, adj = 1, labels = p_labels, xpd = TRUE, cex=1)
dev.off()

# Converting the data.frame into a matrix
matr <- as.matrix(df_s)
# HISTOGRAMS --- a different histogram on a different page
pdfFile <- paste(dir_r,"histograms.pdf", sep="")
pdf(file=pdfFile)
for(i in 1:s_num){
  hist(matr[,i], xlab=p_labels[i], main="")
}
dev.off()

## Visualizing the mean fluorescence values with error bars (standard deviation)
# Defining the barplot error bars
err_bars_pf <- meanr+sigmar
err_bars_mf <- meanr-sigmar

######## BARPLOTS (FULL)
pdfFile <- paste(dir_r,"barplots_full.pdf", sep="")
pdf(file=pdfFile)
# coord is a matrix whose first (and only) column contains the x axis coordinates where the barplots are placed 
coord <- barplot(meanr,xlim=c(0,xlimit),ylim=c(0,ylimit),ylab="Fluorescence (AU)",names.arg=no_names,main=e_name)
# if you plot barplot on a file, you cannot add on it the error bars etc.
errbar(coord[,1],meanr,err_bars_pf,err_bars_mf,add=T,cex=0.25)
text(coord[,1], par("usr")[3], srt = 60, adj = 1, labels = p_labels, xpd = TRUE, cex=1)
dev.off()


#### FILTERING THE DATA, removal of the outliers
matr_flt <- matr
mean_flt <- c()
var_flt <- c()
sigma_flt <- c()

# Converting the matrix columns into vectors, one at each for-loop step 
for(i in 1:s_num){
  sp <- as.vector(matr[,i])
  #flt_lw <- filter_low(sp)
  #flt_sp <- filter(flt_lw)
  mean_flt[i] <- mean(flt_sp,na.rm=TRUE)
  var_flt[i] <- var(flt_sp,na.rm=TRUE)
  sigma_flt[i] <- sqrt(var_flt[i])
  matr_flt[,i] <- flt_sp
}

# Defining the barplot error bars
err_bars_p <- mean_flt+sigma_flt
err_bars_m <- mean_flt-sigma_flt

###### Writing the filtered results into a text file (filter.txt)
wfile <- paste(dir_r,"filter.txt")
write(e_name,wfile,sep="\t",append=TRUE)
for (i in 1:s_num){
  write(colnames(df_s[i]),wfile,sep="\t",append=TRUE)
  write("mean:",wfile,sep="\t",append=TRUE)
  write(mean_flt[i],wfile,sep="\t",append=TRUE)
  write("st.dev.:",wfile,sep="\t",append=TRUE)
  write(sigma_flt[i],wfile,sep="\t",append=TRUE)
  write('',wfile,sep="\t",append=TRUE)
}  
write('sample\tmean\tst.dev.',wfile,append = TRUE)
for (i in 1:s_num){
  lines <- matrix( c(colnames(df_s[i]), mean_flt[i], sigma_flt[i]), nrow=1, ncol=3)
  write(lines, wfile, ncolumns=3, sep="\t",append=TRUE)    
}

## Visualizing the mean fluorescence values with error bars (standard deviation)
######## BARPLOTS
pdfFile <- paste(dir_r,"barplots_filter.pdf", sep="")
pdf(file=pdfFile)
# coord is a matrix whose first (and only) column contains the x axis coordinates where the barplots are placed 
coord <- barplot(mean_flt,xlim=c(0,xlimit),ylim=c(0,ylimit),ylab="Fluorescence (AU)",names.arg=no_names,main=e_name)
# if you plot barplot on a file, you cannot add on it the error bars etc.
errbar(coord[,1],mean_flt,err_bars_p,err_bars_m,add=T,cex=0.25)
text(coord[,1], par("usr")[3], srt = 60, adj = 1, labels = p_labels, xpd = TRUE, cex=1)
dev.off()

# Visualizing the filtered data distribution
# FLT_HISTOGRAMS --- to find a better way
pdfFile <- paste(dir_r,"flt_histograms.pdf", sep="")
pdf(file=pdfFile)
for(i in 1:s_num){
  hist(matr_flt[,i], xlab=p_labels[i], main="")
}
dev.off()
